@@ -1,9 +1,9 @@
 package Win32::IE::Mechanize;
 use strict;
 
-# $Id: Mechanize.pm 225 2005-01-03 22:30:25Z abeltje $
-use vars qw( $VERSION );
-$VERSION = '0.007_4';
+# $Id: Mechanize.pm 389 2005-08-12 17:24:26Z abeltje $
+use vars qw( $VERSION $SLEEP $DEBUG );
+$VERSION = '0.009_15';
 
 =head1 NAME
 
@@ -22,7 +22,7 @@ Win32::IE::Mechanize - Like "the mech" but with IE as user-agent
     $ie->form_name( $form_name );
     $ie->set_fields(
         username => 'yourname',
-        password => 'dummy' 
+        password => 'dummy'
     );
     $ie->click( $btn_name );
 
@@ -33,10 +33,15 @@ Win32::IE::Mechanize - Like "the mech" but with IE as user-agent
             username => 'yourname',
             password => 'dummy',
         },
+        # tick the bar value and untick the baz value
+        # of the foo checkboxes
+        tick      => {
+            foo => { bar => 1, baz => 0 },
+        },
         button    => $btn_name,
     );
 
-Now also trys to support Basic-Authentication like LWP::UserAgent
+Now also tries to support Basic-Authentication like LWP::UserAgent
 
     use Win32::IE::Mechanize;
 
@@ -44,7 +49,7 @@ Now also trys to support Basic-Authentication like LWP::UserAgent
 
     $ie->credentials( 'pause.perl.org:443', 'PAUSE', 'abeltje', '********' );
     $ie->get( 'https://pause.perl.org/pause/authenquery' );
- 
+
 =head1 DESCRIPTION
 
 This module tries to be a sort of drop-in replacement for
@@ -66,12 +71,17 @@ code and nicked most of your documentation!>
 For more information on the OLE2 interface for InternetExplorer, google
 for B<InternetExplorer.Application+msdn>.
 
-=head1 Construction and properties
+=head1 CONSTRUCTION AND PROPERTIES
 
 =cut
 
 use URI;
-use Win32::OLE;
+use Win32::OLE qw( EVENTS );
+
+BEGIN {
+    $DEBUG = 0;
+    Win32::OLE->Option( '_Unique' );
+}
 
 # These are properties of InternetExplorer.Application we support
 my %ie_property = (
@@ -98,28 +108,31 @@ See C<L<set_property()>> for supported options.
 =cut
 
 sub new {
-    my $proto = shift;
-    my $class = ref $proto ? ref $proto : $proto;
+    my $class = shift;
+
+    my $self = bless {
+        _opt       => {},
+        quiet      => 0,
+        readystate => 3,
+        olewarn    => 0,
+        onwarn     => \&Win32::IE::Mechanize::__warn,
+        onerror    => \&Win32::IE::Mechanize::__die,
+    }, $class;
 
-    my $self = {
-        agent => Win32::OLE->new( 'InternetExplorer.Application' ),
-    };
     my %opt = @_ ? UNIVERSAL::isa( $_[0], 'HASH' ) ? %{ $_[0] } : @_ : ();
     $self->{_opt} = { map {
         ( $_ => __prop_value( $_, $opt{ $_ } ) )
     } grep exists $ie_property{ lc $_ } => keys %opt };
-    foreach my $prop ( keys %{ $self->{_opt} } ) {
-        defined $self->{_opt}{ $prop } and
-            $self->{agent}->{ $prop } = $self->{_opt}{ $prop };
-    }
 
     # some more options not for IE
     $self->{ $_ } = exists $opt{ $_ } ? $opt{ $_ } : undef
         for qw( quiet onwarn onerror );
-    $self->{onwarn}  ||= \&Win32::IE::Mechanize::__warn;
-    $self->{onerror} ||= \&Win32::IE::Mechanize::__die;
 
-    bless $self, $class;
+    exists $opt{ $_ } and  $self->{ $_ } = $opt{ $_ }
+        for qw( readystate olewarn );
+
+    Win32::OLE->Option( Warn => $self->{olewarn} );
+    $self->open;
 }
 
 =head2 $ie->set_property( %opt )
@@ -182,8 +195,13 @@ sub set_property {
 
     foreach my $prop ( keys %opt ) {
         defined $opt{ $prop } and
-            $self->{agent}->{ $prop } = $opt{ $prop };
+            $self->{agent}->{ $prop } = $self->{_opt}{ $prop } = $opt{ $prop };
     }
+    if ( $raw{elowarn} ) {
+        $self->{olewarn} = $raw{olewarn};
+        Win32::OLE->Option( Warn => $self->{olewarn} );
+    }
+    return scalar @{[ keys( %opt ), grep /^olewarn$/ => keys %raw ]}
 }
 
 =head2 $ie->close
@@ -194,16 +212,35 @@ Close the InternetExplorer instance.
 
 sub close { $_[0]->{agent}->quit; $_[0]->{agent} = undef; }
 
+=head2 $ie->open()
+
+Create a I<InternetExplorer.Application> OLE object, set all its
+properties and enable the I<DWebBrowserEvents2> event-handle.
+
+=cut
+
 sub open {
     my $self = shift;
     defined $self->{agent} and return;
-    $self->{agent} = Win32::OLE->new( 'InternetExplorer.Application' );
+    $self->{agent} = Win32::OLE->new( 'InternetExplorer.Application' ) or
+        $self->die( "Cannot create an InternetExplorer.Application" );
+
     foreach my $prop ( keys %{ $self->{_opt} } ) {
         defined $self->{_opt}{ $prop } and
             $self->{agent}->{ $prop } = $self->{_opt}{ $prop };
     }
+
+    Win32::OLE->WithEvents( $self->{agent}, \&win32_ie_events,
+                            "DWebBrowserEvents2" );
+    return $self;
 }
 
+=head2 $ie->agent
+
+Return a reference to the InternetExplorer automation object.
+
+=cut
+
 sub agent { $_[0]->{agent} }
 
 =head1 PAGE-FETCHING METHODS
@@ -217,9 +254,9 @@ wait for it to be loaded.
 
 sub get {
     my $self = shift;
-    my $agent = $self->{agent};
+    my $agent = $self->agent;
     my( $url ) = @_;
-     
+
     my $uri = $self->uri 
         ? URI->new_abs( $url, $self->uri->as_string )
         : URI->new( $url );
@@ -236,7 +273,7 @@ loaded.
 =cut
 
 sub reload {
-     $_[0]->{agent}->Refresh;
+     $_[0]->agent->Refresh;
      $_[0]->_wait_while_busy;
 }
 
@@ -248,7 +285,7 @@ loaded.
 =cut
 
 sub back {
-    $_[0]->{agent}->GoBack;
+    $_[0]->agent->GoBack;
     $_[0]->_wait_while_busy;
 }
 
@@ -260,7 +297,7 @@ Return true for ReadyState >= 2;
 
 =cut
 
-sub success { $_[0]->{agent}->ReadyState >= 2 }
+sub success { $_[0]->agent->ReadyState >= $_[0]->{readystate} }
 
 =head2 $ie->uri
 
@@ -268,7 +305,7 @@ Return the URI of this document (as an URI object).
 
 =cut
 
-sub uri { URI->new( $_[0]->{agent}->LocationURL ) }
+sub uri { URI->new( $_[0]->agent->LocationURL ) }
 
 =head2 $ie->ct
 
@@ -277,17 +314,69 @@ return the MIME type in a way we expect.
 
 =cut
 
-sub ct { 
-    my $ct = $_[0]->{agent}->Document->mimeType;
+sub ct {
+    my $ct = $_[0]->agent->Document->mimeType;
     CASE: {
         local $_ = $ct;
-        /^HTML (?:Document|File)/i and return "text/html";
-        /^(\w+) Image/i   and return "text/\L$1";
+        /^HTML/i              and return "text/html";
+        /^Text/i              and return "text/plain";
+        /^(\w+) Image/i       and return "image/\L$1";
+        /^(?:jpe?g|gif|png)/i and return "image/\L$1";
 
-        return $_;
+        return _ct_from_registry( $_ );
     }
 }
 
+=begin private
+
+=head2 _ct_from_registry( $iemime )
+
+This is a bit af a pain:
+
+=over 4
+
+=item * Find a key like C<qr/.+?file/> in HKEY_CLASSES_ROOT with value
+$iemime (put it in @fts)
+
+=item * Find a key like C<qr/\..+/> in HKEY_CLASSES_ROOT with value
+$ft and a subkey I<Content Type>, that value can be used as Content Type.
+
+=back
+
+=end private
+
+=cut
+
+sub _ct_from_registry {
+    my $iemime = shift;
+
+    eval q{use Win32::TieRegistry};
+    $@ and warn( "registry module: $@" ), return $iemime;
+
+    $Win32::TieRegistry::Registry->Delimiter( '/' );
+    my $Classes = $Win32::TieRegistry::Registry->{ qq[Classes/] };
+
+    my @filetypes = grep m|^(.+?file/)$| &&
+                         ($Classes->{ qq[$1/] }||'') eq $iemime
+        => keys %$Classes;
+
+    my $mimeType = $iemime;
+    for my $ft ( @filetypes ) {
+        ( my $ftclean = $ft ) =~ s|/$||;
+        my @exts = grep m|^(\..+?/)$| &&
+                        ($Classes->{ qq[$1/] }||'') eq $ftclean &&
+    		    $Classes->{ qq[${1}Content Type] }
+            => keys %$Classes;
+        for my $ext ( @exts ) {
+            ( my $extclean = $ext ) =~s|^\.(.+?)/$|$1|;
+            $iemime =~ m|\b$extclean\b|i and
+		return $Classes->{ qq[${ext}Content Type] };
+        }
+        @exts and $mimeType = $Classes->{ qq[$exts[0]Content Type] };
+    }
+    return $mimeType;
+}
+
 =head2 $ie->current_form
 
 Returns the current form as an C<Win32::IE::Form> object.
@@ -300,23 +389,6 @@ sub current_form {
     $self->{cur_form};
 }
 
-=head2 $ie->links
-
-When called in a list context, returns a list of the links found in
-the last fetched page. In a scalar context it returns a reference to
-an array with those links. The links returned are all
-C<Win32::IE::Link> objects.
-
-=cut
-
-sub links {
-    my $self = shift;
-
-    defined $self->{links} or $self->{links} = $self->_extract_links;
-
-    return wantarray ? @{ $self->{links} } : $self->{links};
-}
-
 =head2 $ie->is_html
 
 Return true if this is an HTML Document.
@@ -334,7 +406,7 @@ Fetch the C<title> from the C<< $ie->Document >>.
 
 =cut
 
-sub title { $_[0]->{agent}->Document->title }
+sub title { $_[0]->agent->Document->title }
 
 =head1 CONTENT-HANDLING METHODS
 
@@ -348,10 +420,27 @@ and beware all tags are upcased :(
 
 =cut
 
-sub content { $_[0]->{agent}->Document->documentElement->{outerHTML} }
+sub content { $_[0]->agent->Document->documentElement->{outerHTML} }
 
 =head1 LINK METHODS
 
+=head2 $ie->links
+
+When called in a list context, returns a list of the links found in
+the last fetched page. In a scalar context it returns a reference to
+an array with those links. The links returned are all
+C<Win32::IE::Link> objects.
+
+=cut
+
+sub links {
+    my $self = shift;
+
+    defined $self->{links} or $self->{links} = $self->_extract_links;
+
+    return wantarray ? @{ $self->{links} } : $self->{links};
+}
+
 =head2 $ie->follow_link( %opt )
 
 Uses the C<< $self->find_link() >> interface to locate a link and C<<
@@ -619,9 +708,9 @@ key/value pairs:
 
 =item * C<< alt => 'string' >> and C<< alt_regex => qr/regex/, >>
 
-C<alt> matches the ALT attribute of the image against I<string>, which must be a
-n
-exact match. To select a image with an ALT tag that is exactly "download", use
+C<alt> matches the ALT attribute of the image against I<string>, which
+must be an exact match. To select a image with an ALT tag that is
+exactly "download", use
 
     $ie->find_image( alt  => "download" );
 
@@ -655,7 +744,7 @@ The tags supported are C<< <img> >> and C<< <input> >>.
 
 =back
 
-If C<n> is not specified, it defaults to 1.  Therefore, if you don't
+If C<n> is not specified, it defaults to 1.  Therefore, if you do not
 specify any parms, this method defaults to finding the first image on the
 page.
 
@@ -782,7 +871,7 @@ sub form_number {
 
     my $number = shift || 1;
     $self->_extract_forms unless defined $self->{forms};
-    if ( $self->{forms} && $number <= @{ $self->{forms} } ) {
+    if ( @{ $self->{forms} } && $number <= @{ $self->{forms} } ) {
         $self->{cur_form} = $self->{forms}[ $number - 1 ];
     } else {
         $self->warn( "There is no form numbered $number." );
@@ -801,7 +890,7 @@ found.
 
 sub form_name {
     my $self = shift;
-    
+
     my $name = shift or return undef;
     $self->_extract_forms unless defined $self->{forms};
     my @matches = grep $_->name && $_->name eq $name => @{ $self->{forms} };
@@ -877,7 +966,7 @@ sub select {
 This method sets multiple fields of a form. It takes a list of field
 name and value pairs. If there is more than one field with the same
 name, the first one found is set. If you want to select which of the
-duplicate field to set, use a value which is an anonymous array which
+duplicate fields to set, use a value which is an anonymous array which
 has the field value and its number as the 2 elements.
 
         # set the second foo field
@@ -898,7 +987,7 @@ sub set_fields {
     my %opt = @_ ? UNIVERSAL::isa( $_[0], 'HASH' ) ? %{ $_[0] } : @_ : ();
     while ( my( $fname, $value ) = each %opt ) {
         if ( ref $value eq 'ARRAY' ) {
-            my( $input ) = $form->find_input( $fname, undef, $value->[1] );
+            my( $input ) = $form->find_input( $fname, undef, $value->[1]||1 );
             if ( $input ) {
                 $input->value( $value->[0] );
             } else {
@@ -998,13 +1087,14 @@ sub tick {
     my @check_boxes = grep $_->value eq $value
         => $form->find_input( $name, 'checkbox' );
 
-    $self->warn( "No checkbox '$name'  for value '$value' in form." )
+    $self->warn( "No checkbox '$name'  for value '$value' in form." ), return
         unless @check_boxes;
 
     foreach my $check_box ( @check_boxes ) {
         next unless $check_box->value eq $value;
-        ${$check_box}->{checked} = $set;
+        ${$check_box}->{checked} = $set || 0;
     }
+    return 1;
 }
 
 =head2 $ie->untick( $name, $value )
@@ -1040,6 +1130,10 @@ sub value {
     my $number = shift || 1;
 
     my $form = $self->current_form;
+    if ( wantarray ) {
+        my @inputs = $form->find_input( $name );
+        return @inputs ? map $_->value() => @inputs : undef;
+    }
     if ( $number > 1 ) {
         return $form->find_input( $name, undef, $number )->value();
     } else {
@@ -1047,9 +1141,7 @@ sub value {
     }
 }
 
-=head1 Form submission methods
-
-=head2 $ie->click( $button )
+=head2 $ie->click( $button[, $nowait] )
 
 Call the click method on an INPUT object with the name C<$button> Has
 the effect of clicking a button on a form.  The first argument is the
@@ -1059,7 +1151,7 @@ name of the button to be clicked. I have not found a way to set the
 =cut
 
 sub click {
-    my( $self, $button ) = @_;
+    my( $self, $button, $nowait ) = @_;
 
     my $form = $self->current_form;
     
@@ -1070,7 +1162,7 @@ sub click {
       $form->find_input( $button, 'submit' );
 
     $toclick and $toclick->click;
-    $self->_wait_while_busy;
+    $self->_wait_while_busy( $nowait );
 }
 
 =head2 $ie->click_button( %args )
@@ -1099,6 +1191,11 @@ B<NOTE>: Unlike WWW::Mechanize, Win32::IE::Mechanize takes
 all buttonish types of C<< <INPUT type=> >> into accout: B<button>,
 B<image> and B<submit>.
 
+B<NOTE>: Because buttons can be purely scripted in IE, you may need to
+specify an extra argument C<< nowait => 1 >>. This tels
+Win32::IE::Mechanize not to hook into the eventloop in order to
+prevent "hanging".
+
 =cut
 
 sub click_button {
@@ -1106,7 +1203,7 @@ sub click_button {
     my %args = @_;
 
     for ( keys %args ) {
-        if ( !/^(number|name|value)$/ ) {
+        if ( !/^(number|name|value|nowait)$/ ) {
             $self->warn( qq{Unknown click_button_form parameter "$_"} );
         }
     }
@@ -1121,16 +1218,16 @@ sub click_button {
     @buttons or return;
     if ( $args{name} ) {
         $buttons[0]->click;
-        return $self->_wait_while_busy;
+        return $self->_wait_while_busy( $args{nowait} );
     } elsif ( $args{number} ) {
-        @buttons <= $args{number} and return
-        $buttons[ $args{number} - 1 ]->click();
-        return $self->_wait_while_busy;
+        @buttons <= $args{number} and
+            return $buttons[ $args{number} - 1 ]->click();
+        return $self->_wait_while_busy( $args{nowait} );
     } elsif ( $args{value} ) {
         for my $button ( @buttons ) {
             if ( $button->value eq $args{value} ) {
                 $button->click();
-                return $self->_wait_while_busy;
+                return $self->_wait_while_busy( $args{nowait} );
             }
         }
     }
@@ -1180,6 +1277,15 @@ Selects the form named I<name> (calls C<L<form_name()>>)
 
 Sets the field values from the I<fields> hashref (calls C<L<set_fields()>>)
 
+=item * tick => checkboxes
+
+Ticks and unticks the values specified for each checkboxgroup in the
+hashref I<checkboxes>.
+
+    tick => { foo => { bar => 1, baz => 0 } }
+
+This option is not in WWW::Mechanize.
+
 =item * button => button
 
 Clicks on button I<button> (calls C<L<click()>>)
@@ -1214,9 +1320,19 @@ sub submit_form {
     if ( my $fields = $opt{fields} ) {
         if ( ref $fields eq 'HASH' ) {
             $self->set_fields( %{$fields} ) ;
-        } # TODO: What if it's not a hash?  We just ignore it silently?
+        } # TODO: What if it is not a hash?  We just ignore it silently?
     }
 
+    if ( my $chkbox = $opt{tick} ) {
+        if ( ref $chkbox eq 'HASH' ) {
+            for my $cbname ( keys %$chkbox ) {
+                while ( my( $value, $set ) = each %{ $chkbox->{ $cbname } } ) {
+                    $self->tick( $cbname, $value, $set );
+                }
+            }
+        }
+    } # TODO: what if it is not a hash? We just ignore it silently?
+
     if ( $opt{button} ) {
         if ( ref $opt{button} ) {
             $self->click_button( %{ $opt{button} } );
@@ -1232,6 +1348,54 @@ sub submit_form {
 
 =head1 MISCELANEOUS METHODS
 
+=head2 $ie->add_header( name => $value [, name => $value... ] )
+
+Sets HTTP headers for the agent to add or remove from the HTTP request.
+
+    $ie->add_header( Encoding => 'text/klingon' );
+
+If a I<value> is C<undef>, then that header will be removed from any
+future requests.  For example, to never send a Referer header:
+
+    $ie->add_header( Referer => undef );
+
+Returns the number of name/value pairs added.
+
+=cut
+
+sub add_header {
+    my $self = shift;
+    my $npairs = 0;
+
+    while ( @_ ) {
+        my $key = shift;
+        my $value = shift;
+        ++$npairs;
+
+        $self->{headers}{$key} = $value;
+    }
+
+    return $npairs;
+}
+
+=head2 $ie->delete_header( name [, name ... ] )
+
+Removes HTTP headers from the agents list of special headers.
+
+B<NOTE>: This might not work like it does with C<WWW::Mechanize>
+
+=cut
+
+sub delete_header {
+    my $self = shift;
+
+    while ( @_ ) {
+        my $key = shift;
+
+        delete $self->{headers}{$key};
+    }
+}
+
 =head2 $ie->quiet( [$state] )
 
 Allows you to suppress warnings to the screen.
@@ -1252,19 +1416,21 @@ sub quiet {
 
 =head2 $ie->find_frame( $frame_name )
 
-Returns the URL to the source of the frame with C<name eq $frame_name>
+Returns the URL to the source of the frame with C<name eq $frame_name>.
+
+B<NOTE>This method is depricate. Please use C<find_link()>!
 
 =cut
 
 sub find_frame {
     my( $self, $frame ) = @_;
 
-    my $agent = $self->{agent};
+    my $agent = $self->agent;
     my $doc = $agent->Document;
     for ( my $i = 0; $i < $doc->all->length; $i++ ) {
         my $obj = $doc->all( $i );
         next unless $obj && $obj->tagName &&
-                    $obj->tagName eq 'FRAME' &&
+                    $obj->tagName =~ /^I?FRAME$/ &&
                     $obj->name    eq $frame;
 
         return ( URI->new_abs( $obj->src, $doc->URL ) )->as_string;
@@ -1275,6 +1441,8 @@ sub find_frame {
 
 C<< $self->get( $self->find_frame( $frame_name )) >>
 
+B<NOTE>This method is depricated. Please use C<follow_link()>!
+
 =cut
 
 sub load_frame {
@@ -1283,7 +1451,7 @@ sub load_frame {
     $self->get( $self->find_frame( $frame ) );
 }
 
-=head1 LWP compatability methods
+=head1 LWP COMPATABILITY METHODS
 
 =head2 $ie->credentials( $netloc, $realm, $uid, $pass )
 
@@ -1349,41 +1517,7 @@ sub set_realm {
     $self->{basic_authentication}{ $netloc }{__active_realm__} = $realm;
 }
 
-=head2 $ie->add_header( name => $value [, name => $value... ] )
-
-Sets HTTP headers for the agent to add or remove from the HTTP request.
-
-    $mech->add_header( Encoding => 'text/klingon' );
-
-If a I<value> is C<undef>, then that header will be removed from any
-future requests.  For example, to never send a Referer header:
-
-    $mech->add_header( Referer => undef );
-
-If you want to delete a header, use C<delete_header>.
-
-Returns the number of name/value pairs added.
-
-This method was ported from LWWW::Mechanize)
-
-=cut
-
-sub add_header {
-    my $self = shift;
-    my $npairs = 0;
-
-    while ( @_ ) {
-        my $key = shift;
-        my $value = shift;
-        ++$npairs;
-
-        $self->{headers}{$key} = $value;
-    }
-
-    return $npairs;
-}
-
-=head1 Internal-only methods
+=head1 INTERNAL-ONLY METHODS
 
 =head2 DESTROY
 
@@ -1392,7 +1526,7 @@ We need to close the IE-instance, at least when not visible!
 =cut
 
 sub DESTROY {
-    my $agent = shift->{agent};
+    my $agent = $_[0]->{agent};
     $agent && !$agent->{visible} and $agent->quit;
 }
 
@@ -1406,7 +1540,7 @@ that mimics L<HTML::Form>.
 
 sub _extract_forms {
     my $self = shift;
-    my $doc = $self->{agent}->Document;
+    my $doc = $self->agent->Document;
 
     my @forms;
     for ( my $i = 0; $i < $doc->forms->length; $i++ ) {
@@ -1421,7 +1555,7 @@ sub _extract_forms {
 
 The links come from the following:
 
-=over 0
+=over 2
 
 =item "<A HREF=...>"
 
@@ -1437,7 +1571,7 @@ The links come from the following:
 
 sub _extract_links {
     my $self = shift;
-    my $doc = $self->{agent}->Document;
+    my $doc = $self->agent->Document;
 
     my @links;
     for ( my $i = 0; $i < $doc->all->length; $i++ ) {
@@ -1478,27 +1612,86 @@ sub _extract_images {
     return wantarray ? @{ $self->{forms} } : $self->{forms};
 }
 
-=head2 $self->_wait_while_busy()
+{
 
-This is still a mess, but we need to poll IE to see if it is ready
-loading and displaying the page, before we can move on.
+=begin internals
+
+=head1 IEXPLORER READYSTATE
+
+InternetExplorer is a visual browser with scripting capabilities. This
+introduces the problem that content can still be rendered even while
+the actual resource has finished loading. InternetExplorer.Application
+does itself not have a good idea when it is "done", so we have to take
+a guess.
+
+To take a stab at it, we hook into the B<DWebBrowserEvents2> interface
+and keep track of the I<DownloadBegin>/I<DownloadComplete>
+event-pairs. At the end of a loading cycle the I<DocumentComplete>
+event is fired (ReadyState == 4). After that, as it turns out, some
+more events are fired and it looks like the InternetExplorer
+automation object initialisation (including the visible rendering) is
+finished. This seems to end with the I<ProgressChange> event with two
+zero parameters being fired. We take this event to be the signal that
+things are "ready". This is pure speculation and not documented; so it
+might be wrong.
+
+=head2 init_win32_ie_events()
+
+Sets the counters needed to keep track of the I<Download*> events.
+
+=head2 win32_ie_events()
+
+This is the handler used by L<Win32::OLE> to hook into the
+B<DWebBrowserEvents2> interface.
+
+=end internals
+
+=cut
+
+our( $dl_cnt, $dl_tot ) = ( 0, 0 );
+sub init_win32_ie_events {
+    $dl_cnt = 0; $dl_tot = 0;
+    $DEBUG and print "[init_win32_ie_events] ($dl_cnt/$dl_tot)\n";
+}
+
+sub win32_ie_events {
+    my( $agent, $event, @args ) = @_;
+
+    CASE: {
+        $event eq 'DownloadBegin' and do {
+            $dl_cnt++;
+            last CASE;
+        };
+
+        $event eq 'DownloadComplete' and do {
+            $dl_cnt--; $dl_tot++;
+            last CASE;
+        };
+
+        $event eq 'ProgressChange' && $dl_tot && $agent->readyState >= 3 &&
+        $args[0] == 0 && $args[1] == 0 and do {
+#print "[PC $agent->{readyState} $ev_first] @args ($dl_cnt/$dl_tot)\n";
+            Win32::OLE->QuitMessageLoop;
+            last CASE;
+        }
+    }
+    $DEBUG and print "[$event $agent->{readyState}] @args ($dl_cnt/$dl_tot)\n";
+}
+}
+
+=head2 $self->_wait_while_busy( $nowait )
+
+Start hooking into the browser events.
 
 =cut
 
 sub _wait_while_busy {
-    my $self = shift;
+    my( $self, $nowait ) = @_;
     my $agent = $self->{agent};
 
-    # The documentation isn't clear on this.
-    # The DocumentComplete event roughly says:
-    # the event gets fired (for each frame) after ReadyState == 4
-    # we might need to check if the first one has frames
-    # and do some more checking.
-    my $sleep = 0; # 0.4;
-#    while ( $agent->{Busy} == 1 ) { $sleep and sleep( $sleep ) }
-#    return unless $agent->{ReadyState};
-    while ( $agent->{ReadyState} <= 2 ) {
-        $sleep and sleep( $sleep );
+    unless ( $nowait ) {
+        init_win32_ie_events();
+        Win32::OLE->MessageLoop; # The handler call QuitMessageLoop to return
     }
 
     $self->{ $_ } = undef for qw( forms cur_form links images );
@@ -1584,7 +1777,7 @@ sub _extra_headers {
     return $header;
 }
 
-=head1 Internal only non-methods
+=head1 INTERNAL ONLY NON-METHODS
 
 =head2 __prop_value( $key[, $value] )
 
@@ -1650,7 +1843,10 @@ Retuns a new Win32::IE::Image. NOT a method!
 
 =cut
 
+use Win32::IE::Form;
 sub __new_form  { return Win32::IE::Form->new( @_ )  }
+
+use Win32::IE::Input;
 sub __new_input { return Win32::IE::Input->new( @_ ) }
 
 use Win32::IE::Link;
@@ -1661,386 +1857,52 @@ sub __new_image { return Win32::IE::Image->new( @_ ) }
 
 1;
 
-package Win32::IE::Form;
-
-=head1 PACKAGE Win32::IE::Form
-
-Win32::IE::Form - Like <HTML::Form> but for the IE form object.
-
-=head1 METHODS
-
-=head2 Win32::IE::Form->new( $form_obj );
-
-Initialize a new object, it is only a ref to a scalar, the rest is
-done through the methods.
-
-=cut
-
-sub new {
-    my $class = shift;
-    
-    bless \(my $self = shift), $class;
-}
-
-=head2 $form->method( [$new_method] )
-
-Get/Set the I<method> used to submit the from (i.e. B<GET> or
-B<POST>).
-
-=cut
-
-sub method {
-    my $self = shift;
-    my $form = $$self;
-    
-    $form->{method} = shift if  @_;
-    return $form->{method};
-}
-
-=head2 $form->action( [$new_action] )
-
-Get/Set the I<action> for submitting the form.
-
-=cut
-
-sub action {
-    my $self = shift;
-    my $form = $$self;
-    
-    $form->{action} = shift if @_;
-    return $form->{action};
-}
-
-=head2 $form->enctype( [$new_enctype] )
-
-Get/Set the I<enctype> of the form.
-
-=cut
-
-sub enctype {
-    my $self = shift;
-    my $form = $$self;
-
-    $form->{enctype} = shift if @_;
-    return $form->{enctype};
-}
-
-=head2 $form->attr( $name[, $new_value] )
-
-Get/Set any of the attributes from the FORM-tag.
-
-=cut
-
-sub attr {
-    my $self = shift;
-    my $form = $$self;
-
-    return unless @_;
-    my $name = shift;
-    my $index = undef;
-    for (my $i = 0; $i < $form->attributes->length; $i++ ) {
-        next unless $form->attributes( $i )->name eq $name;
-        $index = $i;
-        last;
-    }
-    if ( defined $index ) {
-        $form->attributes( $index )->{value} = shift if @_;
-        return $form->attributes( $index )->{value};
-    } else {
-        return;
-    }
-}
-
-=head2 $form->name()
-
-Return the name of this form.
-
-=cut
-
-sub name {
-    my $self = shift;
-    my $form = $$self;
-
-    return $form->{name}
-}
-
-=head2 $form->inputs()
-
-Returns a list of L<Win32::IE::Input> objects. In scalar context it
-will return the number of inputs.
-
-=cut
-
-sub inputs {
-    my $self = shift;
-    my $form = $$self;
-
-    my $ok_tags = join "|", qw( BUTTON INPUT SELECT TEXTAREA );
-    my( @inputs, %radio_seen ) ;
-    for ( my $i = 0; $i < $form->all->length; $i++ ) {
-        next unless $form->all( $i )->tagName =~ /$ok_tags/i;
-        if ( lc( $form->all( $i )->tagName ) eq 'input' &&
-             lc( $form->all( $i )->type    ) eq 'radio' ) {
-
-            $radio_seen{ $form->all( $i )->name }++ or
-                push @inputs, Win32::IE::Input->new( $form->all( $i ) );
-
-        } else {
-            push @inputs, Win32::IE::Input->new( $form->all( $i ) );
-        }
-    }
-
-    return wantarray ? @inputs : scalar @inputs;
-}
-
-=head2 $form->find_input( $name[, $type[, $index]] )
-
-This method is used to locate specific inputs within the form.  All
-inputs that match the arguments given are returned.  In scalar context
-only the first is returned, or C<undef> if none match.
-
-If $name is specified, then the input must have the indicated name.
-
-If $type is specified, then the input must have the specified type.
-The following type names are used: "text", "password", "hidden",
-"textarea", "file", "image", "submit", "radio", "checkbox" and "option".
-
-The $index is the sequence number of the input matched where 1 is the
-first.  If combined with $name and/or $type then it select the I<n>th
-input with the given name and/or type.
-
-(This method is ported from L<HTML::Form>)
-
-=cut
-
-sub find_input {
-    my $self = shift;
-    my $form = $$self;
-
-    my( $name, $type, $index ) = @_;
-    my $typere = qr/.*/;
-    $type and $typere = $type =~ /^select/i ? qr/^$type/i : qr/^$type$/i; 
-
-    if ( wantarray ) {
-        my( $cnt, @res ) = ( 0 );
-        for my $input ( $self->inputs ) {
-            if ( defined $name ) {
-                $input->name or next;
-                $input->name ne $name and next;
-            }
-            $input->type =~ $typere or next;
-            $cnt++;
-            $index && $index ne $cnt and next;
-            push @res, $input;
-        }
-        return @res;
-    } else {
-        $index ||= 1;
-        for my $input ( $self->inputs ) {
-            if ( defined $name ) {
-                $input->name or next;
-                $input->name ne $name and next;
-            }
-            $input->type =~ $typere or next;
-            --$index and next;
-            return $input;
-        }
-        return undef;
-    }
-}
-
-=head2 $form->value( $name[, $new_value] )
-
-Get/Set the value for the input-contol with specified name.
-
-=cut
-
-sub value {
-    my $self = shift;
-    my $form = $$self;
-
-    my $input = $self->find_input( shift );
-    return $input->value( @_ );
-}
-
-=head2 $form->submit()
-
-Submit this form.
-
-=cut
-
-sub submit {
-    my $self = shift;
-    my $form = $$self;
-
-    $form->submit;
-}
-
-=head2 $self->_radio_group( $name )
-
-Returns a list of Win32::OLE objects with C<< <input type="radio"
-name="$name"> >>.
-
-=cut
-
-sub _radio_group {
-    my $self = shift;
-    my $form = $$self;
-
-    my $name = shift or return;
-    my @rgroup;
-    for ( my $i = 0; $i < $form->all->length; $i++ ) {
-        next unless $form->all( $i )->tagName =~ /input/i;
-        next unless $form->all( $i )->type =~ /radio/i;
-        next unless $form->all( $i )->name eq $name;
-        push @rgroup, $form->all( $i );
-    }
-
-    return wantarray ? @rgroup : \@rgroup;
-}
-
-package Win32::IE::Input;
-
-=head1 PACKAGE Win32::IE::Input
-
-Win32::IE::Input - A small class to interface with the IE input objects.
-
-=head1 METHODS
-
-=head2 Win32::IE::Input->new( $ie_input )
-
-Initialize a new object (like L<Win32::IE::Form>).
-
-=cut
-
-sub new {
-    my $class = shift;
-
-    bless \(my $self = shift), $class;
-}
-
-=head2 $input->name
-
-Return the input-control name.
-
-=cut
-
-sub name { return ${ $_[0] }->name; }
-
-=head2 $input->type
-
-Return the type of the input control.
-
-=cut
-
-sub type { return lc ${ $_[0] }->type; }
-
-=head2 $input->value( [$value] )
-
-Get/Set the value of the input control.
-
-=cut
-
-sub value {
-    my $self = shift;
-    my $input = $$self;
-
-    $self->type =~ /^select/i and return $self->select_value( @_ );
-    $self->type =~ /^radio/i  and return $self->radio_value( @_ );
-
-    $input->{value} = shift if @_ && defined $_[0];
-    return $input->{value};
-}
-
-=head2 $input->select_value( [$value] )
-
-Mark all options from the options collection with C<$value> as
-selected and unselect all other options.
-
-=cut
-
-sub select_value {
-    my $self = shift;
-    my $input = $$self;
-
-    my %vals;
-    if ( @_ ) {
-        my @values = @_;
-        if ( @values == 1 && ref $values[0] eq 'HASH' ) {
-            my @ords = ref $values[0]->{n}
-                ? @{ $values[0]->{n} } : $values[0]->{n};
-            @values = ();
-            foreach my $i ( @ords ) {
-                $i > 0 && $i <= $input->options->{length} and
-                    push @values, $input->options( $i - 1 )->{value};
-            }
-        }
-        @values = @{ $values[0] } if @values == 1 && ref $values[0];
-
-        # Make sure only the last value is set for:
-        # select-one type with multiple values;
-        @values = ( $values[-1] ) if lc( $input->type ) eq 'select-one';
- 
-        %vals = map { ( $_ => undef ) } @values;
-
-        for ( my $i = 0; $i < $input->options->{length}; $i++ ) {
-            $input->options( $i )->{selected} = 
-                exists $vals{ $input->options( $i )->{value} };
-        }
-    } else {
-        for ( my $i = 0; $i < $input->options->{length}; $i++ ) {
-            $input->options( $i )->{selected} and
-                $vals{ $input->options( $i )->{value} } = 1;
-        }
-    }
-    return keys %vals;
-}
-
-=head2 $input->radio_value( [$value] )
-
-Locate all radio-buttons with the same name within this form. Now
-uncheck all values that are not equal to C<$value>.
-
-=cut
-
-sub radio_value {
-    my $self = shift;
-    my $input = $$self;
-
-    my $form = Win32::IE::Form->new( $input->form );
-    my @radios = $form->_radio_group( $self->name );
-
-    if ( @_ ) {
-        my $value = shift;
-        $_->{checked} = $_->value eq $value for @radios;
-    }
-    my( $value ) = map $_->{value} => grep $_->checked => @radios;
-    return $value;
-}
-
-=head2 $input->click
-
-Calls the C<click()> method on the actual object. This may not work.
-
-=cut
-
-sub click { ${ $_[0] }->click }
-
 =head1 CAVEATS
 
 The InternetExplorer automation object does not provide an interface
 to the options menus. This means that you will need to set all options
-needed for this automation before you start.
+needed for this automation before you start. This means that you may
+need to set your security settings to a low and possibly unsafe level.
 
 The InternetExplorer automation object does not provide an interface
-to "popup windows" generated by options or JScript contained in the
-page.
+to "popup windows" generated by security settings or JScript contained
+in the page.
+
+The InternetExplorer automation object does not provide an interface
+to the content of frames from the toplevel document. This means that
+you need to load the frame explicitly (using the follow_link()
+method).
+
+The basic authentication support is quite wonky and will only work
+with the C<get()> method.
 
 =head1 BUGS
 
 Yes, there are bugs. The test-suite doesn't fully cover the codebase.
 
-Please report bugs to L<http://rt.cpan.org>
+=over 4
+
+=item frames
+
+Some reports have come in regarding frames. So far nobody has been
+able to provide me with a testcase, so I can't realy "fix" those.
+
+=item forks
+
+Gabor Szabo reported problems with forking and C<use
+Win32::IE::Mechanize>. His workaround is to change the "global" C<use
+Win32::IE::Mechanize> with localized C<require Win32::IE::Mechanize>
+in the child's code.
+
+=item encodings
+
+Gabor Szabo reported problems with pages that use non-standard
+encoding. In his case it turns out that somewhere along the line, the
+right-to-left hebrew contents are lost.
+
+=back
+
+Please report bugs using L<http://rt.cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
